@@ -1,3 +1,4 @@
+# SPDX-License-Identifier: MIT OR Apache-2.0
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
@@ -6,27 +7,29 @@
 Logger wrapper and helper class.
 """
 
-from __future__ import absolute_import, division, print_function
+from __future__ import annotations
 
-from six import string_types
+from typing import Any, Iterable, Mapping, Sequence
 
 from structlog.exceptions import DropEvent
 
+from .typing import BindableLogger, Context, Processor, WrappedLogger
 
-class BoundLoggerBase(object):
+
+class BoundLoggerBase:
     """
     Immutable context carrier.
 
     Doesn't do any actual logging; examples for useful subclasses are:
 
-        - the generic :class:`BoundLogger` that can wrap anything,
-        - :class:`structlog.twisted.BoundLogger`,
-        - and :class:`structlog.stdlib.BoundLogger`.
+    - the generic `BoundLogger` that can wrap anything,
+    - `structlog.stdlib.BoundLogger`.
+    - `structlog.twisted.BoundLogger`,
 
     See also `custom-wrappers`.
     """
 
-    _logger = None
+    _logger: WrappedLogger
     """
     Wrapped logger.
 
@@ -37,17 +40,22 @@ class BoundLoggerBase(object):
         See also `custom-wrappers`.
     """
 
-    def __init__(self, logger, processors, context):
+    def __init__(
+        self,
+        logger: WrappedLogger,
+        processors: Iterable[Processor],
+        context: Context,
+    ):
         self._logger = logger
         self._processors = processors
         self._context = context
 
-    def __repr__(self):
-        return "<{0}(context={1!r}, processors={2!r})>".format(
+    def __repr__(self) -> str:
+        return "<{}(context={!r}, processors={!r})>".format(
             self.__class__.__name__, self._context, self._processors
         )
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         try:
             if self._context == other._context:
                 return True
@@ -56,14 +64,12 @@ class BoundLoggerBase(object):
         except AttributeError:
             return False
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self.__eq__(other)
 
-    def bind(self, **new_values):
+    def bind(self, **new_values: Any) -> BoundLoggerBase:
         """
         Return a new logger with *new_values* added to the existing ones.
-
-        :rtype: ``self.__class__``
         """
         return self.__class__(
             self._logger,
@@ -71,25 +77,22 @@ class BoundLoggerBase(object):
             self._context.__class__(self._context, **new_values),
         )
 
-    def unbind(self, *keys):
+    def unbind(self, *keys: str) -> BoundLoggerBase:
         """
         Return a new logger with *keys* removed from the context.
 
         :raises KeyError: If the key is not part of the context.
-
-        :rtype: ``self.__class__``
         """
         bl = self.bind()
         for key in keys:
             del bl._context[key]
+
         return bl
 
-    def try_unbind(self, *keys):
+    def try_unbind(self, *keys: str) -> BoundLoggerBase:
         """
         Like :meth:`unbind`, but best effort: missing keys are ignored.
 
-        :rtype: ``self.__class__``
-
         .. versionadded:: 18.2.0
         """
         bl = self.bind()
@@ -98,39 +101,42 @@ class BoundLoggerBase(object):
 
         return bl
 
-    def new(self, **new_values):
+    def new(self, **new_values: Any) -> BoundLoggerBase:
         """
-        Clear context and binds *initial_values* using :func:`bind`.
+        Clear context and binds *initial_values* using `bind`.
 
         Only necessary with dict implementations that keep global state like
         those wrapped by `structlog.threadlocal.wrap_dict` when threads
         are re-used.
-
-        :rtype: ``self.__class__``
         """
         self._context.clear()
+
         return self.bind(**new_values)
 
     # Helper methods for sub-classing concrete BoundLoggers.
 
-    def _process_event(self, method_name, event, event_kw):
+    def _process_event(
+        self, method_name: str, event: str | None, event_kw: dict[str, Any]
+    ) -> tuple[Sequence[Any], Mapping[str, Any]]:
         """
         Combines creates an ``event_dict`` and runs the chain.
 
         Call it to combine your *event* and *context* into an event_dict and
         process using the processor chain.
 
-        :param str method_name: The name of the logger method.  Is passed into
+        :param method_name: The name of the logger method.  Is passed into
             the processors.
         :param event: The event -- usually the first positional argument to a
             logger.
         :param event_kw: Additional event keywords.  For example if someone
-            calls ``log.msg("foo", bar=42)``, *event* would to be ``"foo"``
+            calls ``log.info("foo", bar=42)``, *event* would to be ``"foo"``
             and *event_kw* ``{"bar": 42}``.
+
         :raises: `structlog.DropEvent` if log entry should be dropped.
         :raises: `ValueError` if the final processor doesn't return a
-            string, tuple, or a dict.
-        :rtype: `tuple` of ``(*args, **kw)``
+            str, bytes, bytearray, tuple, or a dict.
+
+        :returns: `tuple` of ``(*args, **kw)``
 
         .. note::
 
@@ -140,29 +146,39 @@ class BoundLoggerBase(object):
 
         .. versionchanged:: 14.0.0
             Allow final processor to return a `dict`.
+        .. versionchanged:: 20.2.0
+            Allow final processor to return `bytes`.
+        .. versionchanged:: 21.2.0
+            Allow final processor to return a `bytearray`.
         """
-        event_dict = self._context.copy()
+        # We're typing it as Any, because processors can return more than an
+        # EventDict.
+        event_dict: Any = self._context.copy()
         event_dict.update(**event_kw)
+
         if event is not None:
             event_dict["event"] = event
         for proc in self._processors:
             event_dict = proc(self._logger, method_name, event_dict)
-        if isinstance(event_dict, string_types):
+
+        if isinstance(event_dict, (str, bytes, bytearray)):
             return (event_dict,), {}
         elif isinstance(event_dict, tuple):
             # In this case we assume that the last processor returned a tuple
             # of ``(args, kwargs)`` and pass it right through.
-            return event_dict
+            return event_dict  # type: ignore[return-value]
         elif isinstance(event_dict, dict):
             return (), event_dict
         else:
             raise ValueError(
-                "Last processor didn't return an appropriate value.  Allowed "
-                "return values are a dict, a tuple of (args, kwargs), or a "
-                "string."
+                "Last processor didn't return an appropriate value.  Valid "
+                "return values are a dict, a tuple of (args, kwargs), bytes, "
+                "or a str."
             )
 
-    def _proxy_to_logger(self, method_name, event=None, **event_kw):
+    def _proxy_to_logger(
+        self, method_name: str, event: str | None = None, **event_kw: Any
+    ) -> Any:
         """
         Run processor chain on event & call *method_name* on wrapped logger.
 
@@ -170,13 +186,13 @@ class BoundLoggerBase(object):
         handling :exc:`structlog.DropEvent`, and finally calls *method_name* on
         :attr:`_logger` with the result.
 
-        :param str method_name: The name of the method that's going to get
+        :param method_name: The name of the method that's going to get
             called.  Technically it should be identical to the method the
             user called because it also get passed into processors.
         :param event: The event -- usually the first positional argument to a
             logger.
         :param event_kw: Additional event keywords.  For example if someone
-            calls ``log.msg("foo", bar=42)``, *event* would to be ``"foo"``
+            calls ``log.info("foo", bar=42)``, *event* would to be ``"foo"``
             and *event_kw* ``{"bar": 42}``.
 
         .. note::
@@ -190,3 +206,21 @@ class BoundLoggerBase(object):
             return getattr(self._logger, method_name)(*args, **kw)
         except DropEvent:
             return
+
+
+def get_context(bound_logger: BindableLogger) -> Context:
+    """
+    Return *bound_logger*'s context.
+
+    The type of *bound_logger* and the type returned depend on your
+    configuration.
+
+    :param bound_logger: The bound logger whose context you want.
+
+    :returns: The *actual* context from *bound_logger*. It is *not* copied
+        first.
+
+    .. versionadded:: 20.2
+    """
+    # This probably will get more complicated in the future.
+    return bound_logger._context
