@@ -1,27 +1,38 @@
+# SPDX-License-Identifier: MIT OR Apache-2.0
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-from __future__ import absolute_import, division, print_function
-
-import collections
+import json
 import logging
 import logging.config
 import os
+import sys
+
+from io import StringIO
+from typing import Any, Callable, Collection, Dict, Optional, Set
 
 import pytest
+import pytest_asyncio
 
-from pretend import call_recorder
+from pretend import call_recorder, stub
 
-from structlog import ReturnLogger, configure, get_logger, reset_defaults
+from structlog import (
+    PrintLogger,
+    ReturnLogger,
+    configure,
+    get_context,
+    reset_defaults,
+)
+from structlog._config import _CONFIG
+from structlog._log_levels import _NAME_TO_LEVEL, CRITICAL, WARN
 from structlog.dev import ConsoleRenderer
 from structlog.exceptions import DropEvent
-from structlog.processors import JSONRenderer
+from structlog.processors import JSONRenderer, KeyValueRenderer
 from structlog.stdlib import (
-    _NAME_TO_LEVEL,
-    CRITICAL,
-    WARN,
+    AsyncBoundLogger,
     BoundLogger,
+    ExtraAdder,
     LoggerFactory,
     PositionalArgumentsFormatter,
     ProcessorFormatter,
@@ -30,11 +41,14 @@ from structlog.stdlib import (
     add_log_level_number,
     add_logger_name,
     filter_by_level,
+    get_logger,
+    recreate_defaults,
     render_to_log_kwargs,
 )
+from structlog.testing import CapturedCall
+from structlog.typing import BindableLogger, EventDict
 
 from .additional_frame import additional_frame
-from .utils import py3_only
 
 
 def build_bl(logger=None, processors=None, context=None):
@@ -51,7 +65,7 @@ def return_method_name(_, method_name, __):
     return method_name
 
 
-class TestLoggerFactory(object):
+class TestLoggerFactory:
     def setup_method(self, method):
         """
         The stdlib logger factory modifies global state to fix caller
@@ -77,12 +91,19 @@ class TestLoggerFactory(object):
         The name guesser walks up the frames until it reaches a frame whose
         name is not from structlog or one of the configurable other names.
         """
+
+        # Compute the names to __main__ so it doesn't get thrown off if people
+        # install plugins that alter the frames. E.g. #370
+        names = set()
+        f = sys._getframe()
+        while f.f_globals["__name__"] != "__main__":
+            names.add(f.f_globals["__name__"].split(".", 1)[0])
+            f = f.f_back
+
         assert (
             "__main__"
             == additional_frame(
-                LoggerFactory(
-                    ignore_frame_names=["tests.", "_pytest.", "pluggy"]
-                )
+                LoggerFactory(ignore_frame_names=list(names))
             ).name
         )
 
@@ -93,14 +114,15 @@ class TestLoggerFactory(object):
         assert file_name == os.path.realpath(__file__)
         assert func_name == "test_deduces_correct_caller"
 
-    @py3_only
     def test_stack_info(self):
+        """
+        If we ask for stack_info, it will returned.
+        """
         logger = _FixedFindCallerLogger("test")
         testing, is_, fun, stack_info = logger.findCaller(stack_info=True)
 
         assert "testing, is_, fun" in stack_info
 
-    @py3_only
     def test_no_stack_info_by_default(self):
         logger = _FixedFindCallerLogger("test")
         testing, is_, fun, stack_info = logger.findCaller()
@@ -119,8 +141,9 @@ class TestLoggerFactory(object):
         assert log_record.filename == os.path.basename(__file__)
 
     def test_sets_correct_logger(self):
-        assert logging.getLoggerClass() is logging.Logger
-
+        """
+        Calling LoggerFactory ensures that Logger.findCaller gets patched.
+        """
         LoggerFactory()
 
         assert logging.getLoggerClass() is _FixedFindCallerLogger
@@ -135,14 +158,21 @@ class TestLoggerFactory(object):
         assert "foo" == lf.name
 
 
-class TestFilterByLevel(object):
+class TestFilterByLevel:
     def test_filters_lower_levels(self):
+        """
+        Log entries below the current level raise a DropEvent.
+        """
         logger = logging.Logger(__name__)
         logger.setLevel(CRITICAL)
+
         with pytest.raises(DropEvent):
             filter_by_level(logger, "warn", {})
 
     def test_passes_higher_levels(self):
+        """
+        Log entries with higher levels are passed through unchanged.
+        """
         logger = logging.Logger(__name__)
         logger.setLevel(WARN)
         event_dict = {"event": "test"}
@@ -152,7 +182,7 @@ class TestFilterByLevel(object):
         assert event_dict is filter_by_level(logger, "exception", event_dict)
 
 
-class TestBoundLogger(object):
+class TestBoundLogger:
     @pytest.mark.parametrize(
         ("method_name"), ["debug", "info", "warning", "error", "critical"]
     )
@@ -174,7 +204,7 @@ class TestBoundLogger(object):
 
     def test_proxies_log(self):
         """
-        BoundLogger.exception.log() is proxied to the apropriate method.
+        BoundLogger.exception.log() is proxied to the appropriate method.
         """
         bl = BoundLogger(ReturnLogger(), [return_method_name], {})
 
@@ -278,8 +308,40 @@ class TestBoundLogger(object):
             "event", exc_info=42
         )
 
+    def test_proxies_bind(self):
+        """
+        Bind calls the correct bind.
+        """
+        bl = build_bl().bind(a=42)
 
-class TestPositionalArgumentsFormatter(object):
+        assert {"a": 42} == get_context(bl)
+
+    def test_proxies_new(self):
+        """
+        Newcalls the correct new.
+        """
+        bl = build_bl().bind(a=42).new(b=23)
+
+        assert {"b": 23} == get_context(bl)
+
+    def test_proxies_unbind(self):
+        """
+        Unbind calls the correct unbind.
+        """
+        bl = build_bl().bind(a=42).unbind("a")
+
+        assert {} == get_context(bl)
+
+    def test_proxies_try_unbind(self):
+        """
+        try_unbind calls the correct try_unbind.
+        """
+        bl = build_bl().bind(a=42).try_unbind("a", "b")
+
+        assert {} == get_context(bl)
+
+
+class TestPositionalArgumentsFormatter:
     def test_formats_tuple(self):
         """
         Positional arguments as simple types are rendered.
@@ -344,7 +406,7 @@ class TestPositionalArgumentsFormatter(object):
         assert {} == formatter(None, None, {"positional_args": ()})
 
 
-class TestAddLogLevelNumber(object):
+class TestAddLogLevelNumber:
     @pytest.mark.parametrize("level, number", _NAME_TO_LEVEL.items())
     def test_log_level_number_added(self, level, number):
         """
@@ -355,7 +417,7 @@ class TestAddLogLevelNumber(object):
         assert number == event_dict["level_number"]
 
 
-class TestAddLogLevel(object):
+class TestAddLogLevel:
     def test_log_level_added(self):
         """
         The log level is added to the event dict.
@@ -373,8 +435,8 @@ class TestAddLogLevel(object):
         assert "warning" == event_dict["level"]
 
 
-@pytest.fixture
-def log_record():
+@pytest.fixture(name="make_log_record")
+def _make_log_record():
     """
     A LogRecord factory.
     """
@@ -395,7 +457,7 @@ def log_record():
     return create_log_record
 
 
-class TestAddLoggerName(object):
+class TestAddLoggerName:
     def test_logger_name_added(self):
         """
         The logger name is added to the event dict.
@@ -406,18 +468,147 @@ class TestAddLoggerName(object):
 
         assert name == event_dict["logger"]
 
-    def test_logger_name_added_with_record(self, log_record):
+    def test_logger_name_added_with_record(self, make_log_record):
         """
         The logger name is deduced from the LogRecord if provided.
         """
         name = "sample-name"
-        record = log_record(name=name)
+        record = make_log_record(name=name)
         event_dict = add_logger_name(None, None, {"_record": record})
 
         assert name == event_dict["logger"]
 
 
-class TestRenderToLogKW(object):
+def extra_dict() -> Dict[str, Any]:
+    """
+    A dict to be passed in the `extra` parameter of the `logging` module's log
+    methods.
+    """
+    return {
+        "this": "is",
+        "some": "extra values",
+        "x_int": 4,
+        "x_bool": True,
+    }
+
+
+@pytest.fixture(name="extra_dict")
+def extra_dict_fixture():
+    return extra_dict()
+
+
+class TestExtraAdder:
+    @pytest.mark.parametrize(
+        "allow, misses",
+        [
+            (None, None),
+            ({}, None),
+            *[({key}, None) for key in extra_dict().keys()],
+            ({"missing"}, {"missing"}),
+            ({"missing", "keys"}, {"missing"}),
+            ({"this", "x_int"}, None),
+        ],
+    )
+    def test_add_extra(
+        self,
+        make_log_record: Callable[[], logging.LogRecord],
+        extra_dict: Dict[str, Any],
+        allow: Optional[Collection[str]],
+        misses: Optional[Set[str]],
+    ):
+        """
+        Extra attributes of a LogRecord object are added to the event dict.
+        """
+        record: logging.LogRecord = make_log_record()
+        record.__dict__.update(extra_dict)
+        event_dict = {"_record": record, "ed_key": "ed_value"}
+        expected = self._copy_allowed(event_dict, extra_dict, allow)
+
+        if allow is None:
+            actual = ExtraAdder()(None, None, event_dict)
+            assert expected == actual
+        actual = ExtraAdder(allow)(None, None, event_dict)
+        assert expected == actual
+        if misses:
+            assert misses.isdisjoint(expected.keys())
+
+    def test_no_record(self):
+        """
+        If the event_dict has no LogRecord, do nothing.
+        """
+        actual = ExtraAdder()(None, None, {})
+
+        assert {} == actual
+
+    @pytest.mark.parametrize(
+        "allow, misses",
+        [
+            (None, None),
+            ({}, None),
+            *[({key}, None) for key in extra_dict().keys()],
+            ({"missing"}, {"missing"}),
+            ({"missing", "keys"}, {"missing"}),
+            ({"this", "x_int"}, None),
+        ],
+    )
+    def test_add_extra_e2e(
+        self,
+        extra_dict: Dict[str, Any],
+        allow: Optional[Collection[str]],
+        misses: Optional[Set[str]],
+    ):
+        """
+        Values passed in the `extra` parameter of the `logging` module's log
+        methods pass through to log output.
+        """
+        logger = logging.Logger(sys._getframe().f_code.co_name)
+        string_io = StringIO()
+        handler = logging.StreamHandler(string_io)
+        formatter = ProcessorFormatter(
+            foreign_pre_chain=[ExtraAdder(allow)],
+            processors=[JSONRenderer()],
+        )
+        handler.setFormatter(formatter)
+        handler.setLevel(0)
+        logger.addHandler(handler)
+        logger.setLevel(0)
+        logging.warning("allow = %s", allow)
+
+        event_dict = {"event": "Some text"}
+        expected = self._copy_allowed(event_dict, extra_dict, allow)
+
+        logger.info("Some %s", "text", extra=extra_dict)
+        actual = {
+            key: value
+            for key, value in json.loads(string_io.getvalue()).items()
+            if not key.startswith("_")
+        }
+
+        assert expected == actual
+        if misses:
+            assert misses.isdisjoint(expected.keys())
+
+    @classmethod
+    def _copy_allowed(
+        cls,
+        event_dict: EventDict,
+        extra_dict: Dict[str, Any],
+        allow: Optional[Collection[str]],
+    ) -> EventDict:
+        if allow is None:
+            return {**event_dict, **extra_dict}
+        else:
+            return {
+                **event_dict,
+                **{
+                    key: value
+                    for key, value in extra_dict.items()
+                    if key in allow
+                },
+            }
+
+
+class TestRenderToLogKW:
     def test_default(self):
         """
         Translates `event` to `msg` and handles otherwise empty `event_dict`s.
@@ -435,9 +626,37 @@ class TestRenderToLogKW(object):
 
         assert {"msg": "message", "extra": event_dict} == d
 
+    def test_handles_special_kw(self, event_dict):
+        """
+        "exc_info", "stack_info", and "stackLevel" aren't passed as extras.
 
-@pytest.fixture
-def configure_for_pf():
+        Cf. https://github.com/hynek/structlog/issues/424
+        """
+        del event_dict["a"]  # needs a repr
+        event_dict["event"] = "message"
+
+        event_dict["exc_info"] = True
+        event_dict["stack_info"] = False
+        event_dict["stackLevel"] = 1
+
+        d = render_to_log_kwargs(None, None, event_dict)
+
+        assert {
+            "msg": "message",
+            "exc_info": True,
+            "stack_info": False,
+            "stackLevel": 1,
+            "extra": {
+                "b": [3, 4],
+                "x": 7,
+                "y": "test",
+                "z": (1, 2),
+            },
+        } == d
+
+
+@pytest.fixture(name="configure_for_processor_formatter")
+def _configure_for_processor_formatter():
     """
     Configure structlog to use ProcessorFormatter.
 
@@ -455,18 +674,36 @@ def configure_for_pf():
     reset_defaults()
 
 
-def configure_logging(pre_chain, logger=None, pass_foreign_args=False):
+def configure_logging(
+    pre_chain,
+    logger=None,
+    pass_foreign_args=False,
+    renderer=ConsoleRenderer(colors=False),
+):
     """
     Configure logging to use ProcessorFormatter.
+
+    Return a list that is filled with event dicts form calls.
     """
-    return logging.config.dictConfig(
+    event_dicts = []
+
+    def capture(_, __, ed):
+        event_dicts.append(ed.copy())
+
+        return ed
+
+    logging.config.dictConfig(
         {
             "version": 1,
             "disable_existing_loggers": False,
             "formatters": {
                 "plain": {
                     "()": ProcessorFormatter,
-                    "processor": ConsoleRenderer(colors=False),
+                    "processors": [
+                        capture,
+                        ProcessorFormatter.remove_processors_meta,
+                        renderer,
+                    ],
                     "foreign_pre_chain": pre_chain,
                     "format": "%(message)s [in %(funcName)s]",
                     "logger": logger,
@@ -490,29 +727,30 @@ def configure_logging(pre_chain, logger=None, pass_foreign_args=False):
         }
     )
 
+    return event_dicts
 
-class TestProcessorFormatter(object):
+
+@pytest.mark.usefixtures("configure_for_processor_formatter")
+class TestProcessorFormatter:
     """
     These are all integration tests because they're all about integration.
     """
 
-    def test_foreign_delegate(self, configure_for_pf, capsys):
+    def test_foreign_delegate(self, capsys):
         """
         If foreign_pre_chain is None, non-structlog log entries are delegated
-        to logging.
+        to logging. The processor chain's event dict is invoked with
+        `_from_structlog=False`
         """
-        configure_logging(None)
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
+        calls = configure_logging(None)
 
         logging.getLogger().warning("foo")
 
         assert ("", "foo [in test_foreign_delegate]\n") == capsys.readouterr()
+        assert calls[0]["_from_structlog"] is False
+        assert isinstance(calls[0]["_record"], logging.LogRecord)
 
-    def test_clears_args(self, configure_for_pf, capsys):
+    def test_clears_args(self, capsys):
         """
         We render our log records before sending it back to logging.  Therefore
         we must clear `LogRecord.args` otherwise the user gets an
@@ -528,20 +766,13 @@ class TestProcessorFormatter(object):
             "hello world. [in test_clears_args]\n",
         ) == capsys.readouterr()
 
-    def test_pass_foreign_args_true_sets_positional_args_key(
-        self, configure_for_pf, capsys
-    ):
+    def test_pass_foreign_args_true_sets_positional_args_key(self):
         """
         If `pass_foreign_args` is `True` we set the `positional_args` key in
         the `event_dict` before clearing args.
         """
         test_processor = call_recorder(lambda l, m, event_dict: event_dict)
         configure_logging((test_processor,), pass_foreign_args=True)
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
 
         positional_args = {"foo": "bar"}
         logging.getLogger().info("okay %(foo)s", positional_args)
@@ -551,9 +782,9 @@ class TestProcessorFormatter(object):
         assert "positional_args" in event_dict
         assert positional_args == event_dict["positional_args"]
 
-    def test_log_dict(self, configure_for_pf, capsys):
+    def test_log_dict(self, capsys):
         """
-        Test that dicts can be logged with std library loggers.
+        dicts can be logged with std library loggers.
         """
         configure_logging(None)
 
@@ -564,17 +795,12 @@ class TestProcessorFormatter(object):
             "{'foo': 'bar'} [in test_log_dict]\n",
         ) == capsys.readouterr()
 
-    def test_foreign_pre_chain(self, configure_for_pf, capsys):
+    def test_foreign_pre_chain(self, capsys):
         """
         If foreign_pre_chain is an iterable, it's used to pre-process
         non-structlog log entries.
         """
-        configure_logging((add_log_level,))
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
+        configure_logging([add_log_level])
 
         logging.getLogger().warning("foo")
 
@@ -583,16 +809,11 @@ class TestProcessorFormatter(object):
             "[warning  ] foo [in test_foreign_pre_chain]\n",
         ) == capsys.readouterr()
 
-    def test_foreign_pre_chain_add_logger_name(self, configure_for_pf, capsys):
+    def test_foreign_pre_chain_add_logger_name(self, capsys):
         """
         foreign_pre_chain works with add_logger_name processor.
         """
         configure_logging((add_logger_name,))
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
 
         logging.getLogger("sample-name").warning("foo")
 
@@ -603,7 +824,7 @@ class TestProcessorFormatter(object):
         ) == capsys.readouterr()
 
     def test_foreign_chain_can_pass_dictionaries_without_excepting(
-        self, configure_for_pf, capsys
+        self, capsys
     ):
         """
         If a foreign logger passes a dictionary to a logging function,
@@ -624,21 +845,16 @@ class TestProcessorFormatter(object):
             "test_foreign_chain_can_pass_dictionaries_without_excepting]\n",
         ) == capsys.readouterr()
 
-    def test_foreign_pre_chain_gets_exc_info(self, configure_for_pf, capsys):
+    def test_foreign_pre_chain_gets_exc_info(self):
         """
         If non-structlog record contains exc_info, foreign_pre_chain functions
         have access to it.
         """
         test_processor = call_recorder(lambda l, m, event_dict: event_dict)
-        configure_logging((test_processor,))
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
+        configure_logging((test_processor,), renderer=KeyValueRenderer())
 
         try:
-            raise RuntimeError("oh noo")
+            raise RuntimeError("oh no")
         except Exception:
             logging.getLogger().exception("okay")
 
@@ -647,9 +863,34 @@ class TestProcessorFormatter(object):
         assert "exc_info" in event_dict
         assert isinstance(event_dict["exc_info"], tuple)
 
-    def test_other_handlers_get_original_record(
-        self, configure_for_pf, capsys
-    ):
+    def test_foreign_pre_chain_sys_exc_info(self):
+        """
+        If a foreign_pre_chain function accesses sys.exc_info(),
+        ProcessorFormatter should not have changed it.
+        """
+
+        class MyException(Exception):
+            pass
+
+        def add_excinfo(logger, log_method, event_dict):
+            event_dict["exc_info"] = sys.exc_info()
+            return event_dict
+
+        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        configure_logging(
+            (add_excinfo, test_processor), renderer=KeyValueRenderer()
+        )
+
+        try:
+            raise MyException("oh no")
+        except Exception:
+            logging.getLogger().error("okay")
+
+        event_dict = test_processor.calls[0].args[2]
+
+        assert MyException is event_dict["exc_info"][0]
+
+    def test_other_handlers_get_original_record(self):
         """
         Logging handlers that come after the handler with ProcessorFormatter
         should receive original, unmodified record.
@@ -658,9 +899,10 @@ class TestProcessorFormatter(object):
 
         handler1 = logging.StreamHandler()
         handler1.setFormatter(ProcessorFormatter(JSONRenderer()))
-        handler2 = type("", (), {})()
-        handler2.handle = call_recorder(lambda record: None)
-        handler2.level = logging.INFO
+        handler2 = stub(
+            handle=call_recorder(lambda record: None),
+            level=logging.INFO,
+        )
         logger = logging.getLogger()
         logger.addHandler(handler1)
         logger.addHandler(handler2)
@@ -674,7 +916,7 @@ class TestProcessorFormatter(object):
         assert "meh" == handler2_record.msg
 
     @pytest.mark.parametrize("keep", [True, False])
-    def test_formatter_unsets_exc_info(self, configure_for_pf, capsys, keep):
+    def test_formatter_unsets_exc_info(self, capsys, keep):
         """
         Stack traces doesn't get printed outside of the json document when
         keep_exc_info are set to False but preserved if set to True.
@@ -683,7 +925,6 @@ class TestProcessorFormatter(object):
         logger = logging.getLogger()
 
         def format_exc_info_fake(logger, name, event_dict):
-            event_dict = collections.OrderedDict(event_dict)
             del event_dict["exc_info"]
             event_dict["exception"] = "Exception!"
             return event_dict
@@ -697,7 +938,7 @@ class TestProcessorFormatter(object):
         logger.handlers[0].setFormatter(formatter)
 
         try:
-            raise RuntimeError("oh noo")
+            raise RuntimeError("oh no")
         except Exception:
             logging.getLogger().exception("seen worse")
 
@@ -713,8 +954,7 @@ class TestProcessorFormatter(object):
             assert "Traceback (most recent call last):" in err
 
     @pytest.mark.parametrize("keep", [True, False])
-    @py3_only
-    def test_formatter_unsets_stack_info(self, configure_for_pf, capsys, keep):
+    def test_formatter_unsets_stack_info(self, capsys, keep):
         """
         Stack traces doesn't get printed outside of the json document when
         keep_stack_info are set to False but preserved if set to True.
@@ -741,11 +981,11 @@ class TestProcessorFormatter(object):
         else:
             assert 2 == err.count("Stack (most recent call last):")
 
-    def test_native(self, configure_for_pf, capsys):
+    def test_native(self, capsys):
         """
         If the log entry comes from structlog, it's unpackaged and processed.
         """
-        configure_logging(None)
+        eds = configure_logging(None)
 
         get_logger().warning("foo")
 
@@ -753,13 +993,31 @@ class TestProcessorFormatter(object):
             "",
             "[warning  ] foo [in test_native]\n",
         ) == capsys.readouterr()
+        assert eds[0]["_from_structlog"] is True
+        assert isinstance(eds[0]["_record"], logging.LogRecord)
 
-    def test_foreign_pre_chain_filter_by_level(self, configure_for_pf, capsys):
+    def test_native_logger(self, capsys):
+        """
+        If the log entry comes from structlog, it's unpackaged and processed.
+        """
+        logger = logging.getLogger()
+        eds = configure_logging(None, logger=logger)
+
+        get_logger().warning("foo")
+
+        assert (
+            "",
+            "[warning  ] foo [in test_native_logger]\n",
+        ) == capsys.readouterr()
+        assert eds[0]["_from_structlog"] is True
+        assert isinstance(eds[0]["_record"], logging.LogRecord)
+
+    def test_foreign_pre_chain_filter_by_level(self, capsys):
         """
         foreign_pre_chain works with filter_by_level processor.
         """
         logger = logging.getLogger()
-        configure_logging((filter_by_level,), logger=logger)
+        configure_logging([filter_by_level], logger=logger)
         configure(
             processors=[ProcessorFormatter.wrap_for_formatter],
             logger_factory=LoggerFactory(),
@@ -772,3 +1030,202 @@ class TestProcessorFormatter(object):
             "",
             "foo [in test_foreign_pre_chain_filter_by_level]\n",
         ) == capsys.readouterr()
+
+    def test_processor_and_processors(self):
+        """
+        Passing both processor and processors raises a TypeError.
+        """
+        with pytest.raises(TypeError, match="mutually exclusive"):
+            ProcessorFormatter(processor=1, processors=[1])
+
+    def test_no_renderer(self):
+        """
+        Passing neither processor nor processors raises a TypeError.
+        """
+        with pytest.raises(TypeError, match="must be passed"):
+            ProcessorFormatter()
+
+    def test_remove_processors_meta(self):
+        """
+        remove_processors_meta removes _record and _from_structlog. And only
+        them.
+        """
+        assert {"foo": "bar"} == ProcessorFormatter.remove_processors_meta(
+            None,
+            None,
+            {"foo": "bar", "_record": "foo", "_from_structlog": True},
+        )
+
+
+@pytest_asyncio.fixture(name="abl")
+async def _abl(cl):
+    return AsyncBoundLogger(cl, context={}, processors=[])
+
+
+@pytest.mark.skipif(
+    sys.version_info[:2] < (3, 7),
+    reason="AsyncBoundLogger is only for Python 3.7 and later.",
+)
+class TestAsyncBoundLogger:
+    def test_sync_bl(self, abl, cl):
+        """
+        AsyncBoungLogger.sync_bl works outside of loops.
+        """
+        abl.sync_bl.info("test")
+
+        assert [
+            CapturedCall(method_name="info", args=(), kwargs={"event": "test"})
+        ] == cl.calls
+
+    @pytest.mark.asyncio
+    async def test_protocol(self, abl):
+        """
+        AsyncBoundLogger is a proper BindableLogger.
+        """
+        assert isinstance(abl, BindableLogger)
+
+    @pytest.mark.asyncio
+    async def test_correct_levels(self, abl, cl, stdlib_log_method):
+        """
+        The proxy methods call the correct upstream methods.
+        """
+        await getattr(abl.bind(foo="bar"), stdlib_log_method)("42")
+
+        aliases = {"exception": "error", "warn": "warning"}
+
+        alias = aliases.get(stdlib_log_method)
+        if alias:
+            expect = alias
+        else:
+            expect = stdlib_log_method
+
+        assert expect == cl.calls[0].method_name
+
+    @pytest.mark.asyncio
+    async def test_log_method(self, abl, cl):
+        """
+        The `log` method is proxied too.
+        """
+        await abl.bind(foo="bar").log(logging.ERROR, "42")
+
+        assert "error" == cl.calls[0].method_name
+
+    @pytest.mark.asyncio
+    async def test_exception(self, abl, cl):
+        """
+        `exception` makes sure 'exc_info" is set, if it's not set already.
+        """
+        try:
+            raise ValueError("omg")
+        except ValueError:
+            await abl.exception("oops")
+
+        ei = cl.calls[0].kwargs["exc_info"]
+
+        assert ValueError is ei[0]
+        assert ("omg",) == ei[1].args
+
+    @pytest.mark.asyncio
+    async def test_exception_do_not_overwrite(self, abl, cl):
+        """
+        `exception` leaves exc_info be, if it's set.
+        """
+        o1 = object()
+        o2 = object()
+        o3 = object()
+
+        try:
+            raise ValueError("omg")
+        except ValueError:
+            await abl.exception("oops", exc_info=(o1, o2, o3))
+
+        ei = cl.calls[0].kwargs["exc_info"]
+        assert (o1, o2, o3) == ei
+
+    @pytest.mark.asyncio
+    async def test_bind_unbind(self, cl):
+        """
+        new/bind/unbind/try_unbind are correctly propagated.
+        """
+        l1 = AsyncBoundLogger(cl, context={}, processors=[])
+
+        l2 = l1.bind(x=42)
+
+        assert l1 is not l2
+        assert l1.sync_bl is not l2.sync_bl
+        assert {} == l1._context
+        assert {"x": 42} == l2._context
+
+        l3 = l2.new(y=23)
+
+        assert l2 is not l3
+        assert l2.sync_bl is not l3.sync_bl
+        assert {"y": 23} == l3._context
+
+        l4 = l3.unbind("y")
+
+        assert {} == l4._context
+        assert l3 is not l4
+
+        # N.B. x isn't bound anymore.
+        l5 = l4.try_unbind("x")
+
+        assert {} == l5._context
+        assert l4 is not l5
+
+    @pytest.mark.asyncio
+    async def test_integration(self, capsys):
+        """
+        Configure and log an actual entry.
+        """
+
+        configure(
+            processors=[add_log_level, JSONRenderer()],
+            logger_factory=PrintLogger,
+            wrapper_class=AsyncBoundLogger,
+            cache_logger_on_first_use=True,
+        )
+
+        logger = get_logger()
+
+        await logger.bind(foo="bar").info("baz", x="42")
+
+        assert {
+            "foo": "bar",
+            "x": "42",
+            "event": "baz",
+            "level": "info",
+        } == json.loads(capsys.readouterr().out)
+
+        reset_defaults()
+
+
+@pytest.mark.parametrize("log_level", [None, 45])
+def test_recreate_defaults(log_level):
+    """
+    Recreate defaults configures structlog and -- if asked -- logging.
+    """
+    reset_defaults()
+
+    logging.basicConfig(
+        stream=sys.stderr,
+        level=1,
+        force=True,
+    )
+
+    recreate_defaults(log_level=log_level)
+
+    assert BoundLogger is _CONFIG.default_wrapper_class
+    assert dict is _CONFIG.default_context_class
+    assert isinstance(_CONFIG.logger_factory, LoggerFactory)
+
+    # 3.7 doesn't have the force keyword and we don't care enough to
+    # re-implement it.
+    if sys.version_info < (3, 8):
+        return
+
+    log = get_logger().bind()
+    if log_level is not None:
+        assert log_level == log.getEffectiveLevel()
+    else:
+        assert 1 == log.getEffectiveLevel()
